@@ -106,20 +106,18 @@ static unsigned int cond_forbids[] = {
     (1<<ACL_WHERE_HELO),
   0,                                               /* condition */
 
-  (1<<ACL_WHERE_AUTH)|(1<<ACL_WHERE_CONNECT)|      /* control */
-    (1<<ACL_WHERE_HELO)|
-    (1<<ACL_WHERE_MAILAUTH)|
-    (1<<ACL_WHERE_ETRN)|(1<<ACL_WHERE_EXPN)|
-    (1<<ACL_WHERE_STARTTLS)|(ACL_WHERE_VRFY),
+  /* Certain types of control are always allowed, so we let it through
+  always and check in the control processing itself */
 
+  0,                                               /* control */
   0,                                               /* delay */
   (1<<ACL_WHERE_NOTSMTP),                          /* dnslists */
 
   (1<<ACL_WHERE_NOTSMTP)|(1<<ACL_WHERE_AUTH)|      /* domains */
     (1<<ACL_WHERE_CONNECT)|(1<<ACL_WHERE_HELO)|
-    (1<<ACL_WHERE_DATA)|
+    (1<<ACL_WHERE_DATA)|(1<<ACL_WHERE_PREDATA)|
     (1<<ACL_WHERE_ETRN)|(1<<ACL_WHERE_EXPN)|
-    (1<<ACL_WHERE_MAILAUTH)|
+    (1<<ACL_WHERE_MAILAUTH)|(1<<ACL_WHERE_QUIT)|
     (1<<ACL_WHERE_MAIL)|(1<<ACL_WHERE_STARTTLS)|
     (1<<ACL_WHERE_VRFY),
 
@@ -130,9 +128,9 @@ static unsigned int cond_forbids[] = {
 
   (1<<ACL_WHERE_NOTSMTP)|(1<<ACL_WHERE_AUTH)|      /* local_parts */
     (1<<ACL_WHERE_CONNECT)|(1<<ACL_WHERE_HELO)|
-    (1<<ACL_WHERE_DATA)|
+    (1<<ACL_WHERE_DATA)|(1<<ACL_WHERE_PREDATA)|
     (1<<ACL_WHERE_ETRN)|(1<<ACL_WHERE_EXPN)|
-    (1<<ACL_WHERE_MAILAUTH)|
+    (1<<ACL_WHERE_MAILAUTH)|(1<<ACL_WHERE_QUIT)|
     (1<<ACL_WHERE_MAIL)|(1<<ACL_WHERE_STARTTLS)|
     (1<<ACL_WHERE_VRFY),
 
@@ -142,21 +140,21 @@ static unsigned int cond_forbids[] = {
 
   (1<<ACL_WHERE_NOTSMTP)|(1<<ACL_WHERE_AUTH)|      /* recipients */
     (1<<ACL_WHERE_CONNECT)|(1<<ACL_WHERE_HELO)|
-    (1<<ACL_WHERE_DATA)|
+    (1<<ACL_WHERE_DATA)|(1<<ACL_WHERE_PREDATA)|
     (1<<ACL_WHERE_ETRN)|(1<<ACL_WHERE_EXPN)|
-    (1<<ACL_WHERE_MAILAUTH)|
+    (1<<ACL_WHERE_MAILAUTH)|(1<<ACL_WHERE_QUIT)|
     (1<<ACL_WHERE_MAIL)|(1<<ACL_WHERE_STARTTLS)|
     (1<<ACL_WHERE_VRFY),
 
   (1<<ACL_WHERE_AUTH)|(1<<ACL_WHERE_CONNECT)|      /* sender_domains */
     (1<<ACL_WHERE_HELO)|
-    (1<<ACL_WHERE_MAILAUTH)|
+    (1<<ACL_WHERE_MAILAUTH)|(1<<ACL_WHERE_QUIT)|
     (1<<ACL_WHERE_ETRN)|(1<<ACL_WHERE_EXPN)|
     (1<<ACL_WHERE_STARTTLS)|(1<<ACL_WHERE_VRFY),
 
   (1<<ACL_WHERE_AUTH)|(1<<ACL_WHERE_CONNECT)|      /* senders */
     (1<<ACL_WHERE_HELO)|
-    (1<<ACL_WHERE_MAILAUTH)|
+    (1<<ACL_WHERE_MAILAUTH)|(1<<ACL_WHERE_QUIT)|
     (1<<ACL_WHERE_ETRN)|(1<<ACL_WHERE_EXPN)|
     (1<<ACL_WHERE_STARTTLS)|(1<<ACL_WHERE_VRFY),
 
@@ -170,6 +168,43 @@ static unsigned int cond_forbids[] = {
 };
 
 
+/* Return values from decode_control() */
+
+enum { CONTROL_ERROR, CONTROL_CASEFUL_LOCAL_PART, CONTROL_CASELOWER_LOCAL_PART,
+  CONTROL_ENFORCE_SYNC, CONTROL_NO_ENFORCE_SYNC, CONTROL_FREEZE,
+  CONTROL_QUEUE_ONLY, CONTROL_SUBMISSION, CONTROL_NO_MULTILINE };
+
+/* Structure listing various control arguments, with their characteristics.
+The maximum "where" value controls the ACLs in which the various controls are
+permitted to occur. Specifying ACL_WHERE_RCPT limits it to just the RCPT ACL;
+specifying ACL_WHERE_NOTSMTP limits it to "message" ACLs. */
+
+typedef struct control_def {
+  uschar *name;
+  int    value;                  /* CONTROL_xxx value */
+  int    where_max;              /* Maximum "where" value */
+  BOOL   has_option;             /* Has /option(s) following */
+} control_def;
+
+static control_def controls_list[] = {
+  { US"caseful_local_part",     CONTROL_CASEFUL_LOCAL_PART,
+    ACL_WHERE_RCPT,    FALSE },
+  { US"caselower_local_part",   CONTROL_CASELOWER_LOCAL_PART,
+    ACL_WHERE_RCPT,    FALSE },
+  { US"enforce_sync",           CONTROL_ENFORCE_SYNC,
+    INT_MAX,           FALSE },
+  { US"freeze",                 CONTROL_FREEZE,
+    ACL_WHERE_NOTSMTP, FALSE },
+  { US"no_enforce_sync",        CONTROL_NO_ENFORCE_SYNC,
+    INT_MAX,           FALSE },
+  { US"no_multiline_responses", CONTROL_NO_MULTILINE,
+    INT_MAX,           FALSE },
+  { US"queue_only",             CONTROL_QUEUE_ONLY,
+    ACL_WHERE_NOTSMTP, FALSE },
+  { US"submission",             CONTROL_SUBMISSION,
+    ACL_WHERE_NOTSMTP, TRUE  }
+  };
+
 /* Enable recursion between acl_check_internal() and acl_check_condition() */
 
 static int acl_check_internal(int, address_item *, uschar *, int, uschar **,
@@ -404,7 +439,6 @@ static void
 acl_warn(int where, uschar *user_message, uschar *log_message)
 {
 int hlen;
-uschar *s;
 
 if (log_message != NULL && log_message != user_message)
   {
@@ -447,8 +481,7 @@ if (user_message == NULL) return;
 /* If this isn't a message ACL, we can't do anything with a user message.
 Log an error. */
 
-if (where != ACL_WHERE_MAIL && where != ACL_WHERE_RCPT &&
-    where != ACL_WHERE_DATA && where != ACL_WHERE_NOTSMTP)
+if (where > ACL_WHERE_NOTSMTP)
   {
   log_write(0, LOG_MAIN|LOG_PANIC, "ACL \"warn\" with \"message\" setting "
     "found in a non-message (%s) ACL: cannot specify header lines here: "
@@ -456,26 +489,14 @@ if (where != ACL_WHERE_MAIL && where != ACL_WHERE_RCPT &&
   return;
   }
 
-/* If the user message isn't a valid header, turn it into one by adding
-X-ACL-Warn: at the front. */
-
-s = user_message;
-
-while (*s != 0)
-  {
-  if (*s == ':' || !isgraph(*s)) break;
-  s++;
-  }
-
-if (*s != ':')
-  user_message = string_sprintf("X-ACL-Warn: %s", user_message);
+/* Treat the user message as a sequence of one or more header lines. */
 
 hlen = Ustrlen(user_message);
 if (hlen > 0)
   {
   uschar *text, *p, *q;
 
-  /* Add newline if not present */
+  /* Add a final newline if not present */
 
   text = ((user_message)[hlen-1] == '\n')? user_message :
     string_sprintf("%s\n", user_message);
@@ -484,6 +505,8 @@ if (hlen > 0)
 
   for (p = q = text; *p != 0; )
     {
+    uschar *s;
+    int newtype = htype_add_bot;
     header_line **hptr = &acl_warn_headers;
 
     /* Find next header line within the string */
@@ -494,13 +517,45 @@ if (hlen > 0)
       if (*(++q) != ' ' && *q != '\t') break;
       }
 
-    hlen = q - p;
+    /* If the line starts with a colon, interpret the instruction for where to
+    add it. This temporarily sets up a new type. */
+
+    if (*p == ':')
+      {
+      if (strncmpic(p, US":after_received:", 16) == 0)
+        {
+        newtype = htype_add_rec;
+        p += 16;
+        }
+      else if (strncmpic(p, US":at_start:", 10) == 0)
+        {
+        newtype = htype_add_top;
+        p += 10;
+        }
+      else if (strncmpic(p, US":at_end:", 8) == 0)
+        {
+        newtype = htype_add_bot;
+        p += 8;
+        }
+      while (*p == ' ' || *p == '\t') p++;
+      }
+
+    /* See if this line starts with a header name, and if not, add X-ACL-Warn:
+    to the front of it. */
+
+    for (s = p; s < q - 1; s++)
+      {
+      if (*s == ':' || !isgraph(*s)) break;
+      }
+
+    s = string_sprintf("%s%.*s", (*s == ':')? "" : "X-ACL-Warn: ", q - p, p);
+    hlen = Ustrlen(s);
 
     /* See if this line has already been added */
 
     while (*hptr != NULL)
       {
-      if (Ustrncmp((*hptr)->text, p, hlen) == 0) break;
+      if (Ustrncmp((*hptr)->text, s, hlen) == 0) break;
       hptr = &((*hptr)->next);
       }
 
@@ -509,11 +564,10 @@ if (hlen > 0)
     if (*hptr == NULL)
       {
       header_line *h = store_get(sizeof(header_line));
-      h->text = string_copyn(p, hlen);
+      h->text = s;
       h->next = NULL;
-      h->type = htype_other;
+      h->type = newtype;
       h->slen = hlen;
-
       *hptr = h;
       hptr = &(h->next);
       }
@@ -614,6 +668,7 @@ acl_verify(int where, address_item *addr, uschar *arg,
 {
 int sep = '/';
 int callout = -1;
+int callout_overall = -1;
 int verify_options = 0;
 int rc;
 BOOL verify_header_sender = FALSE;
@@ -622,6 +677,8 @@ BOOL callout_defer_ok = FALSE;
 BOOL no_details = FALSE;
 address_item *sender_vaddr = NULL;
 uschar *verify_sender_address = NULL;
+uschar *pm_mailfrom = NULL;
+uschar *se_mailfrom = NULL;
 uschar *list = arg;
 uschar *ss = string_nextinlist(&list, &sep, big_buffer, big_buffer_size);
 
@@ -693,13 +750,11 @@ in place of the actual sender (rare special-case requirement). */
 else if (strncmpic(ss, US"sender", 6) == 0)
   {
   uschar *s = ss + 6;
-  if (where != ACL_WHERE_NOTSMTP &&
-      where != ACL_WHERE_MAIL &&
-      where != ACL_WHERE_RCPT &&
-      where != ACL_WHERE_DATA)
+  if (where > ACL_WHERE_NOTSMTP)
     {
     *log_msgptr = string_sprintf("cannot verify sender in ACL for %s "
-      "(only possible for MAIL, RCPT, or DATA)", acl_wherenames[where]);
+      "(only possible for MAIL, RCPT, PREDATA, or DATA)",
+      acl_wherenames[where]);
     return ERROR;
     }
   if (*s == 0)
@@ -741,7 +796,7 @@ while ((ss = string_nextinlist(&list, &sep, big_buffer, big_buffer_size))
 
   else if (strcmpic(ss, US"check_postmaster") == 0)
      {
-     verify_options |= vopt_callout_postmaster;
+     pm_mailfrom = US"";
      if (callout == -1) callout = CALLOUT_TIMEOUT_DEFAULT;
      }
 
@@ -768,13 +823,67 @@ while ((ss = string_nextinlist(&list, &sep, big_buffer, big_buffer_size))
              verify_options |= vopt_callout_no_cache;
           else if (strcmpic(opt, US"random") == 0)
              verify_options |= vopt_callout_random;
-          else if (strcmpic(opt, US"postmaster") == 0)
-             verify_options |= vopt_callout_postmaster;
           else if (strcmpic(opt, US"use_sender") == 0)
              verify_options |= vopt_callout_recipsender;
           else if (strcmpic(opt, US"use_postmaster") == 0)
              verify_options |= vopt_callout_recippmaster;
-          else
+          else if (strcmpic(opt, US"postmaster") == 0) pm_mailfrom = US"";
+
+          else if (strncmpic(opt, US"mailfrom", 8) == 0)
+            {
+            if (!verify_header_sender)
+              {
+              *log_msgptr = string_sprintf("\"mailfrom\" is allowed as a "
+                "callout option only for verify=header_sender (detected in ACL "
+                "condition \"%s\")", arg);
+              return ERROR;
+              }
+            opt += 8;
+            while (isspace(*opt)) opt++;
+            if (*opt++ != '=')
+              {
+              *log_msgptr = string_sprintf("'=' expected after "
+                "\"mailfrom\" in ACL condition \"%s\"", arg);
+              return ERROR;
+              }
+            while (isspace(*opt)) opt++;
+            se_mailfrom = string_copy(opt);
+            }
+
+          else if (strncmpic(opt, US"postmaster_mailfrom", 19) == 0)
+            {
+            opt += 19;
+            while (isspace(*opt)) opt++;
+            if (*opt++ != '=')
+              {
+              *log_msgptr = string_sprintf("'=' expected after "
+                "\"postmaster_mailfrom\" in ACL condition \"%s\"", arg);
+              return ERROR;
+              }
+            while (isspace(*opt)) opt++;
+            pm_mailfrom = string_copy(opt);
+            }
+
+          else if (strncmpic(opt, US"maxwait", 7) == 0)
+            {
+            opt += 7;
+            while (isspace(*opt)) opt++;
+            if (*opt++ != '=')
+              {
+              *log_msgptr = string_sprintf("'=' expected after \"maxwait\" in "
+                "ACL condition \"%s\"", arg);
+              return ERROR;
+              }
+            while (isspace(*opt)) opt++;
+            callout_overall = readconf_readtime(opt, 0, FALSE);
+            if (callout_overall < 0)
+              {
+              *log_msgptr = string_sprintf("bad time value in ACL condition "
+                "\"verify %s\"", arg);
+              return ERROR;
+              }
+            }
+          else    /* Plain time is callout connect/command timeout */
             {
             callout = readconf_readtime(opt, 0, FALSE);
             if (callout < 0)
@@ -818,7 +927,8 @@ message if giving out verification details. */
 
 if (verify_header_sender)
   {
-  rc = verify_check_header_address(user_msgptr, log_msgptr, callout);
+  rc = verify_check_header_address(user_msgptr, log_msgptr, callout,
+    callout_overall, se_mailfrom, pm_mailfrom, verify_options);
   if (smtp_return_error_details)
     {
     if (*user_msgptr == NULL && *log_msgptr != NULL)
@@ -898,7 +1008,8 @@ else if (verify_sender_address != NULL)
       /* The recipient, qualify, and expn options are never set in
       verify_options. */
 
-      rc = verify_address(sender_vaddr, NULL, verify_options, callout, &routed);
+      rc = verify_address(sender_vaddr, NULL, verify_options, callout,
+        callout_overall, se_mailfrom, pm_mailfrom, &routed);
 
       HDEBUG(D_acl) debug_printf("----------- end verify ------------\n");
 
@@ -941,7 +1052,7 @@ else
 
   addr2 = *addr;
   rc = verify_address(&addr2, NULL, verify_options|vopt_is_recipient, callout,
-    NULL);
+    callout_overall, se_mailfrom, pm_mailfrom, NULL);
   HDEBUG(D_acl) debug_printf("----------- end verify ------------\n");
   *log_msgptr = addr2.message;
   *user_msgptr = addr2.user_message;
@@ -1006,6 +1117,55 @@ return ERROR;
 
 
 
+/*************************************************
+*        Check argument for control= modifier    *
+*************************************************/
+
+/* Called from acl_check_condition() below
+
+Arguments:
+  arg         the argument string for control=
+  pptr        set to point to the terminating character
+  where       which ACL we are in
+  log_msgptr  for error messages
+
+Returns:      CONTROL_xxx value
+*/
+
+static int
+decode_control(uschar *arg, uschar **pptr, int where, uschar **log_msgptr)
+{
+int len;
+control_def *d;
+
+for (d = controls_list;
+     d < controls_list + sizeof(controls_list)/sizeof(control_def);
+     d++)
+  {
+  len = Ustrlen(d->name);
+  if (Ustrncmp(d->name, arg, len) == 0) break;
+  }
+
+if (d >= controls_list + sizeof(controls_list)/sizeof(control_def) ||
+   (arg[len] != 0 && (!d->has_option || arg[len] != '/')))
+  {
+  *log_msgptr = string_sprintf("syntax error in \"control=%s\"", arg);
+  return CONTROL_ERROR;
+  }
+
+if (where > d->where_max)
+  {
+  *log_msgptr = string_sprintf("cannot use \"control=%s\" in %s ACL",
+    arg, acl_wherenames[where]);
+  return CONTROL_ERROR;
+  }
+
+*pptr = arg + len;
+return d->value;
+}
+
+
+
 /*************************************************
 *   Handle conditions/modifiers on an ACL item   *
 *************************************************/
@@ -1042,6 +1202,7 @@ acl_check_condition(int verb, acl_condition_block *cb, int where,
 {
 uschar *user_message = NULL;
 uschar *log_message = NULL;
+uschar *p;
 int rc = OK;
 
 for (; cb != NULL; cb = cb->next)
@@ -1162,44 +1323,53 @@ for (; cb != NULL; cb = cb->next)
     break;
 
     case ACLC_CONTROL:
-    if (where != ACL_WHERE_MAIL && where != ACL_WHERE_RCPT &&
-        where != ACL_WHERE_DATA && where != ACL_WHERE_NOTSMTP)
+    switch (decode_control(arg, &p, where, log_msgptr))
       {
-      *log_msgptr = string_sprintf("cannot use \"control\" in %s ACL",
-        acl_wherenames[where]);
+      case CONTROL_ERROR:
       return ERROR;
-      }
-    if (Ustrcmp(arg, "freeze") == 0)
-      {
+
+      case CONTROL_CASEFUL_LOCAL_PART:
+      deliver_localpart = addr->cc_local_part;
+      break;
+
+      case CONTROL_CASELOWER_LOCAL_PART:
+      deliver_localpart = addr->lc_local_part;
+      break;
+
+      case CONTROL_ENFORCE_SYNC:
+      smtp_enforce_sync = TRUE;
+      break;
+
+      case CONTROL_NO_ENFORCE_SYNC:
+      smtp_enforce_sync = FALSE;
+      break;
+
+      case CONTROL_NO_MULTILINE:
+      no_multiline_responses = TRUE;
+      break;
+
+      case CONTROL_FREEZE:
       deliver_freeze = TRUE;
       deliver_frozen_at = time(NULL);
-      }
-    else if (Ustrcmp(arg, "queue_only") == 0)
-      {
+      break;
+
+      case CONTROL_QUEUE_ONLY:
       queue_only_policy = TRUE;
-      }
-    else if (Ustrncmp(arg, "submission", 10) == 0)
-      {
-
-debug_printf("+++submission arg=%s\n", arg);
+      break;
 
+      case CONTROL_SUBMISSION:
       submission_mode = TRUE;
-      if (Ustrncmp(arg+10, "/domain=", 8) == 0)
+      if (Ustrncmp(p, "/domain=", 8) == 0)
         {
-        submission_domain = string_copy(arg+18);
+        submission_domain = string_copy(p+8);
         }
-      else if (arg[10] != 0)
+      else if (*p != 0)
         {
         *log_msgptr = string_sprintf("syntax error in argument for "
           "\"control\" modifier \"%s\"", arg);
         return ERROR;
         }
-      }
-    else
-      {
-      *log_msgptr = string_sprintf("syntax error in argument for \"control\" "
-        "modifier: \"%s\" is unknown", arg);
-      return ERROR;
+      break;
       }
     break;
 
@@ -1367,7 +1537,9 @@ debug_printf("+++submission arg=%s\n", arg);
 /* If the result is the one for which "message" and/or "log_message" are used,
 handle the values of these options. Most verbs have but a single return for
 which the messages are relevant, but for "discard", it's useful to have the log
-message both when it succeeds and when it fails.
+message both when it succeeds and when it fails. Also, for an "accept" that
+appears in a QUIT ACL, we want to handle the user message. Since only "accept"
+and "warn" are permitted in that ACL, we don't need to test the verb.
 
 These modifiers act in different ways:
 
@@ -1380,7 +1552,8 @@ message that is already set.
 If there isn't a log message set, we make it the same as the user message. */
 
 if (((rc == FAIL_DROP)? FAIL : rc) == msgcond[verb] ||
-    (verb == ACL_DISCARD && rc == OK))
+    (verb == ACL_DISCARD && rc == OK) ||
+    (where == ACL_WHERE_QUIT))
   {
   uschar *expmessage;
 
@@ -1702,11 +1875,20 @@ while (acl != NULL)
   int basic_errno = 0;
   BOOL endpass_seen = FALSE;
 
-  HDEBUG(D_acl) debug_printf("processing \"%s\"\n", verbs[acl->verb]);
-
   *log_msgptr = *user_msgptr = NULL;
   acl_temp_details = FALSE;
 
+  if (where == ACL_WHERE_QUIT &&
+      acl->verb != ACL_ACCEPT &&
+      acl->verb != ACL_WARN)
+    {
+    *log_msgptr = string_sprintf("\"%s\" is not allowed in a QUIT ACL",
+      verbs[acl->verb]);
+    return ERROR;
+    }
+
+  HDEBUG(D_acl) debug_printf("processing \"%s\"\n", verbs[acl->verb]);
+
   /* Clear out any search error message from a previous check before testing
   this condition. */
 
@@ -1892,14 +2074,12 @@ rc = acl_check_internal(where, addr, s, 0, user_msgptr, log_msgptr);
 smtp_command_argument = deliver_domain =
   deliver_localpart = deliver_address_data = NULL;
 
-/* A DISCARD response is permitted only for message ACLs */
+/* A DISCARD response is permitted only for message ACLs, excluding the PREDATA
+ACL, which is really in the middle of an SMTP command. */
 
 if (rc == DISCARD)
   {
-  if (where != ACL_WHERE_MAIL &&
-      where != ACL_WHERE_RCPT &&
-      where != ACL_WHERE_DATA &&
-      where != ACL_WHERE_NOTSMTP)
+  if (where > ACL_WHERE_NOTSMTP || where == ACL_WHERE_PREDATA)
     {
     log_write(0, LOG_MAIN|LOG_PANIC, "\"discard\" verb not allowed in %s "
       "ACL", acl_wherenames[where]);
