@@ -363,7 +363,7 @@ if (running_in_test_harness)
   if (Ustrcmp(endname - 14, "test.again.dns") == 0)
     {
     int delay = Uatoi(name);  /* digits at the start of the name */
-    DEBUG(D_dns) debug_printf("DNS lookup of %s (%s) bypassed for testing\n",
+    DEBUG(D_dns) debug_printf("Real DNS lookup of %s (%s) bypassed for testing\n",
       name, dns_text_type(type));
     if (delay > 0)
       {
@@ -375,7 +375,7 @@ if (running_in_test_harness)
     }
   if (Ustrcmp(endname - 13, "test.fail.dns") == 0)
     {
-    DEBUG(D_dns) debug_printf("DNS lookup of %s (%s) bypassed for testing\n",
+    DEBUG(D_dns) debug_printf("Real DNS lookup of %s (%s) bypassed for testing\n",
       name, dns_text_type(type));
     DEBUG(D_dns) debug_printf("returning DNS_FAIL\n");
     return dns_return(name, type, DNS_FAIL);
