@@ -503,6 +503,10 @@ set to NULL for those that are not compiled into the binary. */
 #include "auths/cram_md5.h"
 #endif
 
+#ifdef AUTH_CYRUS_SASL
+#include "auths/cyrus_sasl.h"
+#endif
+
 #ifdef AUTH_PLAINTEXT
 #include "auths/plaintext.h"
 #endif
@@ -528,6 +532,19 @@ auth_info auths_available[] = {
   },
 #endif
 
+#ifdef AUTH_CYRUS_SASL
+  {
+  US"cyrus_sasl",			     /* lookup name */
+  auth_cyrus_sasl_options,
+  &auth_cyrus_sasl_options_count,
+  &auth_cyrus_sasl_option_defaults,
+  sizeof(auth_cyrus_sasl_options_block),
+  auth_cyrus_sasl_init,                      /* init function */
+  auth_cyrus_sasl_server,                    /* server function */
+  NULL                                       /* client function */
+  },
+#endif
+
 #ifdef AUTH_PLAINTEXT
   {
   US"plaintext",                             /* lookup name */
