@@ -56,6 +56,8 @@ them. Also, the tls_ variables are now always visible. */
 extern int     tls_active;             /* fd/socket when in a TLS session */
 extern BOOL    tls_certificate_verified; /* Client certificate verified */
 extern uschar *tls_cipher;             /* Cipher used */
+extern BOOL    tls_on_connect;         /* For older MTAs that don't STARTTLS */
+extern uschar *tls_on_connect_ports;   /* Ports always tls-on-connect */
 extern uschar *tls_peerdn;             /* DN from peer */
 
 #ifdef SUPPORT_TLS
@@ -65,7 +67,6 @@ extern uschar *tls_certificate;        /* Certificate file */
 extern uschar *tls_crl;                /* CRL File */
 extern uschar *tls_dhparam;            /* DH param file */
 extern BOOL    tls_offered;            /* Server offered TLS */
-extern BOOL    tls_on_connect;         /* For older MTAs that don't STARTTLS */
 extern uschar *tls_privatekey;         /* Private key file */
 extern BOOL    tls_remember_esmtp;     /* For YAEB */
 extern uschar *tls_require_ciphers;    /* So some can be avoided */
@@ -95,17 +96,19 @@ extern uschar **address_expansions[ADDRESS_EXPANSIONS_COUNT];
 extern BOOL    accept_8bitmime;        /* Allow *BITMIME incoming */
 extern tree_node *acl_anchor;          /* Tree of named ACLs */
 extern uschar *acl_not_smtp;           /* ACL run for non-SMTP messages */
-extern uschar *acl_smtp_auth;          /* ACL run after AUTH */
+extern uschar *acl_smtp_auth;          /* ACL run for AUTH */
 extern uschar *acl_smtp_connect;       /* ACL run on SMTP connection */
-extern uschar *acl_smtp_data;          /* ACL run after DATA */
-extern uschar *acl_smtp_etrn;          /* ACL run after ETRN */
-extern uschar *acl_smtp_expn;          /* ACL run after EXPN */
-extern uschar *acl_smtp_helo;          /* ACL run after HELO/EHLO */
-extern uschar *acl_smtp_mail;          /* ACL run after MAIL */
-extern uschar *acl_smtp_mailauth;      /* ACL run after MAIL AUTH */
-extern uschar *acl_smtp_rcpt;          /* ACL run after RCPT */
-extern uschar *acl_smtp_starttls;      /* ACL run after STARTTLS */
-extern uschar *acl_smtp_vrfy;          /* ACL run after VRFY */
+extern uschar *acl_smtp_data;          /* ACL run after DATA received */
+extern uschar *acl_smtp_etrn;          /* ACL run for ETRN */
+extern uschar *acl_smtp_expn;          /* ACL run for EXPN */
+extern uschar *acl_smtp_helo;          /* ACL run for HELO/EHLO */
+extern uschar *acl_smtp_mail;          /* ACL run for MAIL */
+extern uschar *acl_smtp_mailauth;      /* ACL run for MAIL AUTH */
+extern uschar *acl_smtp_predata;       /* ACL run for DATA command */
+extern uschar *acl_smtp_quit;          /* ACL run for QUIT */
+extern uschar *acl_smtp_rcpt;          /* ACL run for RCPT */
+extern uschar *acl_smtp_starttls;      /* ACL run for STARTTLS */
+extern uschar *acl_smtp_vrfy;          /* ACL run for VRFY */
 extern BOOL    acl_temp_details;       /* TRUE to give details for 4xx error */
 extern uschar *acl_var[ACL_C_MAX+ACL_M_MAX]; /* User variables */
 extern uschar *acl_verify_message;     /* User message for verify failure */
@@ -301,6 +304,7 @@ extern uschar *host_number_string;     /* For expanding */
 extern uschar *host_reject_connection; /* Reject these hosts */
 extern tree_node *hostlist_anchor;     /* Tree of defined host lists */
 extern int     hostlist_count;         /* Number defined */
+extern uschar *hosts_connection_nolog; /* Limits the logging option */
 extern uschar *hosts_treat_as_local;   /* For routing */
 
 extern int     ignore_bounce_errors_after; /* Keep them for this time. */
@@ -365,8 +369,10 @@ extern int     message_size;           /* Size of message */
 extern uschar *message_size_limit;     /* As it says */
 extern uschar  message_subdir[];       /* Subdirectory for messages */
 extern uschar *message_reference;      /* Reference for error messages */
+extern BOOL    mua_wrapper;            /* TRUE when Exim is wrapping an MUA */
 
 extern uid_t  *never_users;            /* List of uids never to be used */
+extern BOOL    no_multiline_responses; /* For broken clients */
 
 extern optionlist optionlist_auths[];      /* These option lists are made */
 extern int     optionlist_auths_size;      /* global so that readconf can */
