@@ -327,25 +327,27 @@ return NULL;
 forwarding list, or to run a filter file and get addresses from there.
 
 Arguments:
-  rdata               the redirection block
-  options             the options bits
-  include_directory   restrain to this directory
-  generated           where to hang generated addresses
-  error               for error messages
-  eblockp             for details of skipped syntax errors (NULL => no skip)
-  filtertype          set to the filter type:
-                        FILTER_FORWARD => a traditional .forward file
-                        FILTER_EXIM    => an Exim filter file
-                        FILTER_SIEVE   => a Sieve filter file
-                      a system filter is always forced to be FILTER_EXIM
+  rdata                     the redirection block
+  options                   the options bits
+  include_directory         restrain to this directory
+  sieve_vacation_directory  passed to sieve_interpret
+  generated                 where to hang generated addresses
+  error                     for error messages
+  eblockp                   for details of skipped syntax errors
+                              (NULL => no skip)
+  filtertype                set to the filter type:
+                              FILTER_FORWARD => a traditional .forward file
+                              FILTER_EXIM    => an Exim filter file
+                              FILTER_SIEVE   => a Sieve filter file
+                            a system filter is always forced to be FILTER_EXIM
 
-Returns:              a suitable return for rda_interpret()
+Returns:                    a suitable return for rda_interpret()
 */
 
 static int
 rda_extract(redirect_block *rdata, int options, uschar *include_directory,
-  address_item **generated, uschar **error, error_block **eblockp,
-  int *filtertype)
+  uschar *sieve_vacation_directory, address_item **generated, uschar **error,
+  error_block **eblockp, int *filtertype)
 {
 uschar *data;
 
@@ -360,8 +362,9 @@ else data = rdata->string;
 *filtertype = system_filtering? FILTER_EXIM : rda_is_filter(data);
 
 /* Filter interpretation is done by a general function that is also called from
-the filter testing option (-bf). Several features of string expansion may be
-locked out at sites that don't trust users. This is done by setting flags in
+the filter testing option (-bf). There are two versions: one for Exim filtering
+and one for Sieve filtering. Several features of string expansion may be locked
+out at sites that don't trust users. This is done by setting flags in
 expand_forbid that the expander inspects. */
 
 if (*filtertype != FILTER_FORWARD)
@@ -381,8 +384,12 @@ if (*filtertype != FILTER_FORWARD)
   expand_forbid =
     (expand_forbid & ~RDO_FILTER_EXPANSIONS) |
     (options & RDO_FILTER_EXPANSIONS);
-  frc = ((*filtertype == FILTER_EXIM)? filter_interpret : sieve_interpret)
-    (data, options, generated, error);
+
+  frc = (*filtertype == FILTER_EXIM)?
+    filter_interpret(data, options, generated, error)
+    :
+    sieve_interpret(data, options, sieve_vacation_directory, generated, error);
+
   expand_forbid = old_expand_forbid;
   return frc;
   }
@@ -482,21 +489,22 @@ sub-process is NOT used when:
     process (which is usually running as root).
 
 Arguments:
-  rdata              redirect data (file + constraints, or data string)
-  options            options to pass to the extraction functions,
-                       plus ENOTDIR and EACCES handling bits
-  include_directory  restrain :include: to this directory
-  ugid               uid/gid to run under - if NULL, no change
-  generated          where to hang generated addresses, initially NULL
-  error              pointer for error message
-  eblockp            for skipped syntax errors; NULL if no skipping
-  filtertype         set to the type of file:
-                       FILTER_FORWARD => traditional .forward file
-                       FILTER_EXIM    => an Exim filter file
-                       FILTER_SIEVE   => a Sieve filter file
-                     a system filter is always forced to be FILTER_EXIM
-  rname              router name for error messages in the format
-                       "xxx router" or "system filter"
+  rdata                     redirect data (file + constraints, or data string)
+  options                   options to pass to the extraction functions,
+                              plus ENOTDIR and EACCES handling bits
+  include_directory         restrain :include: to this directory
+  sieve_vacation_directory  directory passed to sieve_interpret()
+  ugid                      uid/gid to run under - if NULL, no change
+  generated                 where to hang generated addresses, initially NULL
+  error                     pointer for error message
+  eblockp                   for skipped syntax errors; NULL if no skipping
+  filtertype                set to the type of file:
+                              FILTER_FORWARD => traditional .forward file
+                              FILTER_EXIM    => an Exim filter file
+                              FILTER_SIEVE   => a Sieve filter file
+                            a system filter is always forced to be FILTER_EXIM
+  rname                     router name for error messages in the format
+                              "xxx router" or "system filter"
 
 Returns:        values from extraction function, or FF_NONEXIST:
                   FF_DELIVERED     success, a significant action was taken
@@ -512,8 +520,8 @@ Returns:        values from extraction function, or FF_NONEXIST:
 
 int
 rda_interpret(redirect_block *rdata, int options, uschar *include_directory,
-  ugid_block *ugid, address_item **generated, uschar **error,
-  error_block **eblockp, int *filtertype, uschar *rname)
+  uschar *sieve_vacation_directory, ugid_block *ugid, address_item **generated,
+  uschar **error, error_block **eblockp, int *filtertype, uschar *rname)
 {
 int fd, rc, pfd[2];
 int yield, status;
@@ -556,8 +564,8 @@ if (!ugid->uid_set ||                         /* Either there's no uid, or */
      rda_is_filter(data) == FILTER_FORWARD && /* It's not a filter script, */
      Ustrstr(data, ":include:") == NULL))     /* and there's no :include: */
   {
-  return rda_extract(rdata, options, include_directory, generated, error,
-    eblockp, filtertype);
+  return rda_extract(rdata, options, include_directory,
+    sieve_vacation_directory, generated, error, eblockp, filtertype);
   }
 
 /* We need to run the processing code in a sub-process. However, if we can
@@ -601,8 +609,8 @@ if ((pid = fork()) == 0)
 
   /* Now do the business */
 
-  yield = rda_extract(rdata, options, include_directory, generated, error,
-    eblockp, filtertype);
+  yield = rda_extract(rdata, options, include_directory,
+    sieve_vacation_directory, generated, error, eblockp, filtertype);
 
   /* Pass back whether it was a filter, and the return code and any overall
   error text via the pipe. */
