@@ -14,6 +14,8 @@ typedef struct {
   BOOL rewrite_headers;
   uschar *widen_domains;
   uschar *mx_domains;
+  uschar *mx_fail_domains;
+  uschar *srv_fail_domains;
   uschar *check_srv;
 } dnslookup_router_options_block;
 
