@@ -351,7 +351,8 @@ for (;;)
     while (*t != 0)
       {
       while (*t != 0 && *t != '\n') t++;
-      debug_printf("  %s %*s\n", (s == ptr)? "LMTP<<" : "      ", t-s, s);
+      debug_printf("  %s %*s\n", (s == ptr)? "LMTP<<" : "      ",
+        (int)(t-s), s);
       if (*t == 0) break;
       s = t = t + 1;
       }
@@ -583,7 +584,12 @@ for (addr = addrlist; addr != NULL; addr = addr->next)
     if (errno != 0 || buffer[0] == 0) goto RESPONSE_FAILED;
     addr->message = string_sprintf("LMTP error after %s: %s", big_buffer,
       string_printing(buffer));
-    if (buffer[0] == '5') addr->transport_return = FAIL;
+    if (buffer[0] == '5') addr->transport_return = FAIL; else
+      {
+      int bincode = (buffer[1] - '0')*10 + buffer[2] - '0';
+      addr->basic_errno = ERRNO_RCPT4XX;
+      addr->more_errno |= bincode << 8;
+      }
     }
   }
 
@@ -684,7 +690,7 @@ RESPONSE_FAILED:
 save_errno = errno;
 addrlist->message = NULL;
 
-if(check_response(&save_errno, addrlist->more_errno,
+if (check_response(&save_errno, addrlist->more_errno,
     buffer, &code, &(addrlist->message)))
   {
   (void) lmtp_write_command(fd_in, "QUIT\r\n");
