@@ -113,35 +113,36 @@ a host list, and a callout has been requested. Callouts are expensive; that is
 why a cache is used to improve the efficiency.
 
 Arguments:
-  addr           the address that's been routed
-  host_list      the list of hosts to try
-  tf             the transport feedback block
+  addr              the address that's been routed
+  host_list         the list of hosts to try
+  tf                the transport feedback block
 
-  ifstring       "interface" option from transport, or NULL
-  portstring     "port" option from transport, or NULL
-  protocolstring "protocol" option from transport, or NULL
-  callout        the callout timeout
-  options        the verification options - these bits are used:
-                   vopt_is_recipient => this is a recipient address
-                   vopt_callout_postmaster => do postmaster check
-                   vopt_callout_no_cache => don't use callout cache
-                   vopt_callout_random => do the "random" thing
-                   vopt_callout_recipsender => use real sender for recipient
-                   vopt_callout_recippmaster => use postmaster for recipient
+  ifstring          "interface" option from transport, or NULL
+  portstring        "port" option from transport, or NULL
+  protocolstring    "protocol" option from transport, or NULL
+  callout           the per-command callout timeout
+  callout_overall   the overall callout timeout (if < 0; use 4*callout)
+  options           the verification options - these bits are used:
+                      vopt_is_recipient => this is a recipient address
+                      vopt_callout_no_cache => don't use callout cache
+                      vopt_callout_random => do the "random" thing
+                      vopt_callout_recipsender => use real sender for recipient
+                      vopt_callout_recippmaster => use postmaster for recipient
+  se_mailfrom         MAIL FROM address for sender verify; NULL => ""
+  pm_mailfrom         if non-NULL, do the postmaster check with this sender
 
-Returns:         OK/FAIL/DEFER
+Returns:            OK/FAIL/DEFER
 */
 
 static int
 do_callout(address_item *addr, host_item *host_list, transport_feedback *tf,
-  int callout, int options)
+  int callout, int callout_overall, int options, uschar *se_mailfrom,
+  uschar *pm_mailfrom)
 {
 BOOL is_recipient = (options & vopt_is_recipient) != 0;
-BOOL callout_postmaster = (options & vopt_callout_postmaster) != 0;
 BOOL callout_no_cache = (options & vopt_callout_no_cache) != 0;
 BOOL callout_random = (options & vopt_callout_random) != 0;
 
-
 int yield = OK;
 BOOL done = FALSE;
 uschar *address_key;
@@ -152,6 +153,7 @@ open_db *dbm_file = NULL;
 dbdata_callout_cache new_domain_record;
 dbdata_callout_cache_address new_address_record;
 host_item *host;
+time_t callout_start_time;
 
 new_domain_record.result = ccache_unknown;
 new_domain_record.postmaster_result = ccache_unknown;
@@ -181,6 +183,16 @@ if (is_recipient)
     }
   }
 
+/* For a sender callout, we must adjust the key if the mailfrom address is not
+empty. */
+
+else
+  {
+  from_address = (se_mailfrom == NULL)? US"" : se_mailfrom;
+  if (from_address[0] != 0)
+    address_key = string_sprintf("%s/<%s>", addr->address, from_address);
+  }
+
 /* Open the callout cache database, it it exists, for reading only at this
 stage, unless caching has been disabled. */
 
@@ -258,7 +270,7 @@ if (dbm_file != NULL)
     but has not been done before, we are going to have to do a callout, so skip
     remaining cache processing. */
 
-    if (callout_postmaster)
+    if (pm_mailfrom != NULL)
       {
       if (cache_record->postmaster_result == ccache_reject)
         {
@@ -279,13 +291,14 @@ if (dbm_file != NULL)
         goto END_CACHE;
         }
 
-      /* If cache says OK, set callout_postmaster FALSE to prevent it
-      trying again if the address has to be checked. Also ensure that
-      the value in the cache record is preserved (with its old timestamp). */
+      /* If cache says OK, set pm_mailfrom NULL to prevent a redundant
+      postmaster check if the address itself has to be checked. Also ensure
+      that the value in the cache record is preserved (with its old timestamp).
+      */
 
       HDEBUG(D_verify) debug_printf("callout cache: domain accepts RCPT "
         "TO:<postmaster@domain>\n");
-      callout_postmaster = FALSE;
+      pm_mailfrom = NULL;
       new_domain_record.postmaster_result = ccache_accept;
       new_domain_record.postmaster_stamp = cache_record->postmaster_stamp;
       }
@@ -313,6 +326,7 @@ if (dbm_file != NULL)
       {
       HDEBUG(D_verify)
         debug_printf("callout cache: address record is negative\n");
+      addr->user_message = US"Previous (cached) callout verification failure";
       yield = FAIL;
       }
     goto END_CALLOUT;
@@ -339,6 +353,12 @@ if (callout_random && callout_random_local_part != NULL)
       "callout_random_local_part: %s", expand_string_message);
   }
 
+/* Default the overall callout timeout if not set, and record the time we are
+starting so that we can enforce it. */
+
+if (callout_overall < 0) callout_overall = 4 * callout;
+callout_start_time = time(NULL);
+
 /* Now make connections to the hosts and do real callouts. The list of hosts
 is passed in as an argument. */
 
@@ -366,6 +386,14 @@ for (host = host_list; host != NULL && !done; host = host->next)
     continue;
     }
 
+  /* Check the overall callout timeout */
+
+  if (time(NULL) - callout_start_time >= callout_overall)
+    {
+    HDEBUG(D_verify) debug_printf("overall timeout for callout exceeded\n");
+    break;
+    }
+
   /* Set IPv4 or IPv6 */
 
   host_af = (Ustrchr(host->address, ':') == NULL)? AF_INET:AF_INET6;
@@ -522,14 +550,15 @@ for (host = host_list; host != NULL && !done; host = host->next)
 
       /* Do postmaster check if requested */
 
-      if (done && callout_postmaster)
+      if (done && pm_mailfrom != NULL)
         {
         done =
           smtp_write_command(&outblock, FALSE, "RSET\r\n") >= 0 &&
           smtp_read_response(&inblock, responsebuffer,
             sizeof(responsebuffer), '2', callout) &&
 
-          smtp_write_command(&outblock, FALSE, "MAIL FROM:<>\r\n") >= 0 &&
+          smtp_write_command(&outblock, FALSE,
+            "MAIL FROM:<%s>\r\n", pm_mailfrom) >= 0 &&
           smtp_read_response(&inblock, responsebuffer,
             sizeof(responsebuffer), '2', callout) &&
 
@@ -594,7 +623,7 @@ for (host = host_list; host != NULL && !done; host = host->next)
 
   /* End the SMTP conversation and close the connection. */
 
-  smtp_write_command(&outblock, FALSE, "QUIT\r\n");
+  (void)smtp_write_command(&outblock, FALSE, "QUIT\r\n");
   close(inblock.sock);
   }    /* Loop through all hosts, while !done */
 
@@ -729,40 +758,47 @@ return yield;
 address testing (-bt), which is indicated by address_test_mode being set.
 
 Arguments:
-  vaddr           contains the address to verify; the next field in this block
-                    must be NULL
-  f               if not NULL, write the result to this file
-  options         various option bits:
-                    vopt_fake_sender => this sender verify is not for the real
-                      sender (it was verify=sender=xxxx) - rewriting does not
-                      change sender_address
-                    vopt_is_recipient => this is a recipient address, otherwise
-                      it's a sender address - this affects qualification and
-                      rewriting and messages from callouts
-                    vopt_qualify => qualify an unqualified address; else error
-                    vopt_expn => called from SMTP EXPN command
+  vaddr            contains the address to verify; the next field in this block
+                     must be NULL
+  f                if not NULL, write the result to this file
+  options          various option bits:
+                     vopt_fake_sender => this sender verify is not for the real
+                       sender (it was verify=sender=xxxx or an address from a
+                       header line) - rewriting must not change sender_address
+                     vopt_is_recipient => this is a recipient address, otherwise
+                       it's a sender address - this affects qualification and
+                       rewriting and messages from callouts
+                     vopt_qualify => qualify an unqualified address; else error
+                     vopt_expn => called from SMTP EXPN command
 
-                    These ones are used by do_callout() -- the options variable
-                      is passed to it.
+                     These ones are used by do_callout() -- the options variable
+                       is passed to it.
 
-                    vopt_callout_postmaster => do postmaster check
-                    vopt_callout_no_cache => don't use callout cache
-                    vopt_callout_random => do the "random" thing
-                    vopt_callout_recipsender => use real sender for recipient
-                    vopt_callout_recippmaster => use postmaster for recipient
+                     vopt_callout_no_cache => don't use callout cache
+                     vopt_callout_random => do the "random" thing
+                     vopt_callout_recipsender => use real sender for recipient
+                     vopt_callout_recippmaster => use postmaster for recipient
 
-  callout         if > 0, specifies that callout is required, and gives timeout
-  routed          if not NULL, set TRUE if routing succeeded, so we can
-                    distinguish between routing failed and callout failed
+  callout          if > 0, specifies that callout is required, and gives timeout
+                     for individual connections and commands
+  callout_overall  if > 0, gives overall timeout for the callout function;
+                   if < 0, a default is used (see do_callout())
+  se_mailfrom      when callout is requested to verify a sender, use this
+                     in MAIL FROM; NULL => ""
+  pm_mailfrom      when callout is requested, if non-NULL, do the postmaster
+                     thing and use this as the sender address (may be "")
 
-Returns:          OK      address verified
-                  FAIL    address failed to verify
-                  DEFER   can't tell at present
+  routed           if not NULL, set TRUE if routing succeeded, so we can
+                     distinguish between routing failed and callout failed
+
+Returns:           OK      address verified
+                   FAIL    address failed to verify
+                   DEFER   can't tell at present
 */
 
 int
 verify_address(address_item *vaddr, FILE *f, int options, int callout,
-  BOOL *routed)
+  int callout_overall, uschar *se_mailfrom, uschar *pm_mailfrom, BOOL *routed)
 {
 BOOL allok = TRUE;
 BOOL full_info = (f == NULL)? FALSE : (debug_selector != 0);
@@ -990,9 +1026,13 @@ while (addr_new != NULL)
               if (tf.gethostbyname || string_is_ip_address(host->name, NULL))
                 (void)host_find_byname(host, NULL, &canonical_name, TRUE);
               else
-                (void)host_find_bydns(host, NULL, HOST_FIND_BY_A, NULL,
-                  tf.qualify_single, tf.search_parents,
+                {
+                int flags = HOST_FIND_BY_A;
+                if (tf.qualify_single) flags |= HOST_FIND_QUALIFY_SINGLE;
+                if (tf.search_parents) flags |= HOST_FIND_SEARCH_PARENTS;
+                (void)host_find_bydns(host, NULL, flags, NULL, NULL, NULL,
                   &canonical_name, NULL);
+                }
               }
             }
           }
@@ -1011,7 +1051,8 @@ while (addr_new != NULL)
           }
         else
           {
-          rc = do_callout(addr, host_list, &tf, callout, options);
+          rc = do_callout(addr, host_list, &tf, callout, callout_overall,
+            options, se_mailfrom, pm_mailfrom);
           }
         }
       else
@@ -1379,20 +1420,25 @@ especially if they are resent- fields, check all of them if there is more than
 one.
 
 Arguments:
-  user_msgptr   points to where to put a user error message
-  log_msgptr    points to where to put a log error message
-  callout       timeout for callout check (passed to verify_address())
+  user_msgptr      points to where to put a user error message
+  log_msgptr       points to where to put a log error message
+  callout          timeout for callout check (passed to verify_address())
+  callout_overall  overall callout timeout (ditto)
+  se_mailfrom      mailfrom for verify; NULL => ""
+  pm_mailfrom      sender for pm callout check (passed to verify_address())
+  options          callout options (passed to verify_address())
 
 If log_msgptr is set to something without setting user_msgptr, the caller
 normally uses log_msgptr for both things.
 
-Returns:        result of the verification attempt: OK, FAIL, or DEFER;
-                FAIL is given if no appropriate headers are found
+Returns:           result of the verification attempt: OK, FAIL, or DEFER;
+                   FAIL is given if no appropriate headers are found
 */
 
 int
 verify_check_header_address(uschar **user_msgptr, uschar **log_msgptr,
-  int callout)
+  int callout, int callout_overall, uschar *se_mailfrom, uschar *pm_mailfrom,
+  int options)
 {
 static int header_types[] = { htype_sender, htype_reply_to, htype_from };
 int yield = FAIL;
@@ -1428,7 +1474,7 @@ for (i = 0; i < 3; i++)
       *ss = 0;
 
       HDEBUG(D_verify) debug_printf("verifying %.*s header address %s\n",
-        endname - h->text, h->text, s);
+        (int)(endname - h->text), h->text, s);
 
       /* See if we have already verified this address as an envelope sender,
       and if so, use the previous answer. */
@@ -1480,7 +1526,8 @@ for (i = 0; i < 3; i++)
         else
           {
           vaddr = deliver_make_addr(address, FALSE);
-          new_ok = verify_address(vaddr, NULL, vopt_fake_sender, callout, NULL);
+          new_ok = verify_address(vaddr, NULL, options | vopt_fake_sender,
+            callout, callout_overall, se_mailfrom, pm_mailfrom, NULL);
           }
         }
 
